#
# Christmas Logic 
# Copyright (C) Jack Whitham 2009
#

# Special things provided by surrounding environment:
# * Create_String
# * Create_Component
# * Meta_Data

from constants import *
from logic import *
import random

class String:
    def __init__(self, name, colour, 
                fill_pos = None, sequence = None,
                image_pos = None, image_name = None):
        self.lights = []
        self.colour = colour
        self.name = name
        self.goal_sequence = []
        self.image_pos = image_pos
        self.image_name = image_name
        self.fill_pos = fill_pos

        if sequence != None:
            self.Add_Sequence(sequence)

    def Add_Light(self, *lights):
        for (x, y) in lights:
            self.lights.append((x, y))

    def Add_Sequence(self, seq):
        for ch in seq:
            if ch == "0":
                self.goal_sequence.append(ST_ZERO)
            elif ch == "1":
                self.goal_sequence.append(ST_ONE)
            else:
                assert False, ch

class Guide:
    def __init__(self, trigger, 
                    point_at = "None", 
                    text = "", requires_ack = False):

        assert type(trigger) == str
        assert type(point_at) == str

        self.trigger_expression = trigger
        self.point_at_expression = point_at
        self.text = text
        self.requires_ack = requires_ack
        self.received_ack = False


def Gen_Flakes(Create_String, seed):
    r = random.Random(seed)

    def Fuzz():
        return r.randint(0, 21) - 10

    def FRow(y):
        return [ (x + 30 + Fuzz(), y + Fuzz()) for
                    x in xrange(0, 241, 60) ]
        
    one = Create_String(String, "Top", colour = (180, 0, 0),
                image_pos = FRow(70), image_name = "flake")
    two = Create_String(String, "Mid", colour = (0, 180, 180),
                image_pos = FRow(150), image_name = "flake")
    three = Create_String(String, "Btm", colour = (180, 0, 180),
                image_pos = FRow(230), image_name = "flake")
    return (one, two, three)

def Gen_Stars(Create_String):
    white = Create_String(String, "white", colour = (255, 255, 255),
                image_name = "whitestar", 
                image_pos = [(50, 50), (250, 50), (150, 150), 
                            (50, 250), (250, 250)])
    blue = Create_String(String, "blue", colour = (120, 120, 255),
                image_name = "bluestar", 
                image_pos = [(150, 50), (50, 150), (250, 150), (150, 250)])

    return (white, blue)

def Gen_SMC(Create_String):
    santa = Create_String(String, "Santa", colour = (255, 0, 0),
                image_pos = [(60, 150)], image_name = "santa")
    merry = Create_String(String, "Mry", colour = (180, 180, 0),
                image_pos = [(200, 150)], image_name = "merry")
    christmas = Create_String(String, "Xms", colour = (180, 180, 180),
                image_pos = [(200, 150)], image_name = "christmas")
    return (santa, merry, christmas)

def Gen_Sledge(Create_String):
    one = Create_String(String, "R", colour = (180, 0, 0),
                image_pos = [(250, 100)], image_name = "sledge")
    two = Create_String(String, "M", colour = (0, 180, 180),
                image_pos = [(150, 150)], image_name = "sledge")
    three = Create_String(String, "L", colour = (180, 0, 180),
                image_pos = [(50, 200)], image_name = "sledge")
    return (one, two, three)

def Gen_Binary(Create_String):
    one = Create_String(String, "One", colour = (255, 255, 0),
                image_pos = [(200, 150)], image_name = "bin")
    two = Create_String(String, "Two", colour = (120, 120, 255),
                image_pos = [(100, 150)], image_name = "bin")
    return (one, two)

def Gen_Chewy(Create_String):
    heart = [
            (110, 214), (89, 186), (73, 155), (52, 122),
            (64, 83), (96, 78), (127, 79), (163, 82),
            (181, 114), (167, 144), (144, 183), (123, 214),
            ]

    def H(offset):
        return [ heart[ i ] for i in xrange(offset, len(heart), 4) ]

    R = Create_String(String, "R", colour = (255, 50, 50),
            fill_pos = [(11,130)] + H(0))
    G = Create_String(String, "G", colour = (0, 255, 0),
            fill_pos = [(217,125)] + H(1))
    B = Create_String(String, "B", colour = (120, 120, 255),
            fill_pos = [(238,129)] + H(2))
    Y = Create_String(String, "Y", colour = (255, 255, 0),
            fill_pos = [(269,128)] + H(3))
    return (R, G, B, Y)

def Gen_Splash(Create_String):
    (one, two, three) = Gen_Flakes(Create_String, 5)

    one.Add_Sequence("100100")
    two.Add_Sequence("010010")
    three.Add_Sequence("001001")
    merry = Create_String(String, "Mry", colour = (180, 180, 0),
                image_pos = [(150, 150)], image_name = "merry")
    christmas = Create_String(String, "Xms", colour = (180, 180, 180),
                image_pos = [(150, 150)], image_name = "christmas")
    merry.Add_Sequence("111000")
    christmas.Add_Sequence("000111")

    return (one, two, three, merry, christmas)

def Mix_Tool(search_list_A, search_list_B, min_count):
    count = 0
    for element_B in search_list_B:
        for element_A in search_list_A:
            if (element_A in element_B.provides):
                count += 1

    return count >= min_count

