#
# Christmas Logic 
# Copyright (C) Jack Whitham 2009
#

import math

def Distance((x1, y1), (x2, y2)):
    x = (x2 - x1) ** 2
    y = (y2 - y1) ** 2
    return math.sqrt(x + y)

def Find_P4(p1, p2, p3):
    d12 = Distance(p1, p2)
    if d12 == 0.0:
        return p1

    d23 = Distance(p2, p3)
    d13 = Distance(p1, p3)
    d24 = ((d23 ** 2) + (d12 ** 2) - (d13 ** 2)) / (2.0 * d12)
    if d24 <= 0:
        # p4 is beyond p2
        return p2
    if d24 >= d12:
        # p4 is beyond p1
        return p1
    
    # p4 is somewhere on the line (p1, p2)
    pos = d24 / d12
    (x1, y1) = p1
    (x2, y2) = p2
    x4 = x2 - (pos * (x2 - x1))
    y4 = y2 - (pos * (y2 - y1))
    return (x4, y4)

def Distance_From_Line(p1, p2, p3):
    return Distance(p3, Find_P4(p1, p2, p3))

def Test_P4():
    from random import random, seed

    seed(221)
    for i in xrange(100000):
        x1 = random() * 10
        y1 = random() * 10
        theta = random() * 2 * math.pi
        d12 = random() * 20

        x2 = x1 + (math.cos(theta) * d12)
        y2 = y1 + (math.sin(theta) * d12)

        d14 = d12 * ((1.2 * random()) - 0.1)
        x4 = x1 + (math.cos(theta) * d14)
        y4 = y1 + (math.sin(theta) * d14)
        p4 = (x4, y4) 
        if d14 <= 0.0:
            p4 = (x1, y1) 
        elif d14 >= d12:
            p4 = (x2, y2) 

        phi = theta - (math.pi * 0.5)
        if random() < 0.5:
            phi += math.pi

        d34 = 50 * random()
        x3 = x4 + (math.cos(phi) * d34)
        y3 = y4 + (math.sin(phi) * d34)
        
        p4a = Find_P4((x1, y1), (x2, y2), (x3, y3))
        error = Distance(p4a, p4)
        assert abs(error) < 1e-6, error

        if 0.0 <= d14 <= d12:
            d34a = Distance_From_Line((x1, y1), (x2, y2), (x3, y3))
            error = d34a - d34
            assert abs(error) < 1e-6, error
    


